@@ -24,6 +24,8 @@
 #include <stdlib.h>
 #include <X11/keysym.h>
 
+#include <pango/pango.h>
+
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 
@@ -53,6 +55,8 @@ const gchar GKBD_INDICATOR_CONFIG_KEY_FOREGROUND_COLOR[] =
 const gchar GKBD_INDICATOR_CONFIG_KEY_BACKGROUND_COLOR[] =
     GKBD_INDICATOR_CONFIG_KEY_PREFIX "/backgroundColor";
 
+#define SYSTEM_FONT_GCONF_ENTRY "/desktop/gnome/interface/font_name"
+
 /**
  * static applet config functions
  */
@@ -74,6 +78,107 @@ gkbd_indicator_config_free_enabled_plugins (GkbdIndicatorConfig *
 	}
 }
 
+static void
+gkbd_indicator_config_load_font_from_gconf (GkbdIndicatorConfig *
+					    ind_config)
+{
+	GError *gerror = NULL;
+
+	ind_config->font_family =
+	    gconf_client_get_string (ind_config->conf_client,
+				     GKBD_INDICATOR_CONFIG_KEY_FONT_FAMILY,
+				     &gerror);
+	if (gerror != NULL) {
+		g_warning ("Error reading configuration:%s\n",
+			   gerror->message);
+		ind_config->font_family = g_strdup ("Helvetica");
+		g_error_free (gerror);
+		gerror = NULL;
+	}
+
+	ind_config->font_size =
+	    gconf_client_get_int (ind_config->conf_client,
+				  GKBD_INDICATOR_CONFIG_KEY_FONT_SIZE,
+				  &gerror);
+	if (gerror != NULL) {
+		g_warning ("Error reading configuration:%s\n",
+			   gerror->message);
+		ind_config->font_size = 10;
+		g_error_free (gerror);
+		gerror = NULL;
+	}
+
+	if (ind_config->font_family == NULL ||
+	    ind_config->font_family[0] == '\0') {
+		PangoFontDescription *fd;
+		gchar *sysfontname =
+		    gconf_client_get_string (ind_config->conf_client,
+					     SYSTEM_FONT_GCONF_ENTRY,
+					     &gerror);
+		if (gerror != NULL) {
+			g_warning ("Error reading configuration:%s\n",
+				   gerror->message);
+			sysfontname = g_strdup ("Sans 10");
+			g_error_free (gerror);
+			gerror = NULL;
+		}
+		fd = pango_font_description_from_string (sysfontname);
+		if (fd != NULL) {
+			ind_config->font_family =
+			    g_strdup (pango_font_description_get_family
+				      (fd));
+			ind_config->font_size =
+			    pango_font_description_get_size (fd) /
+			    PANGO_SCALE;
+			pango_font_description_free (fd);
+		}
+		g_free (sysfontname);
+	}
+	xkl_debug (150, "font: [%s], size %d\n", ind_config->font_family,
+		   ind_config->font_size);
+
+	ind_config->foreground_color =
+	    gconf_client_get_string (ind_config->conf_client,
+				     GKBD_INDICATOR_CONFIG_KEY_FOREGROUND_COLOR,
+				     &gerror);
+	if (gerror != NULL) {
+		g_warning ("Error reading configuration:%s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		gerror = NULL;
+	}
+
+	ind_config->background_color =
+	    gconf_client_get_string (ind_config->conf_client,
+				     GKBD_INDICATOR_CONFIG_KEY_BACKGROUND_COLOR,
+				     &gerror);
+	if (gerror != NULL) {
+		g_warning ("Error reading configuration:%s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		gerror = NULL;
+	}
+
+	if (ind_config->foreground_color == NULL ||
+	    ind_config->foreground_color[0] == '\0') {
+		GtkSettings *settings = gtk_settings_get_default ();
+		GtkStyle *style = gtk_rc_get_style_by_paths (settings,
+							     "*.GtkLabel",
+							     "*.GtkLabel",
+							     GTK_TYPE_LABEL);
+		ind_config->foreground_color =
+		    g_strdup_printf ("%lg %lg %lg",
+				     ((double) style->fg[GTK_STATE_NORMAL].
+				      red) / 0x10000,
+				     ((double) style->fg[GTK_STATE_NORMAL].
+				      green) / 0x10000,
+				     ((double) style->fg[GTK_STATE_NORMAL].
+				      blue) / 0x10000);
+
+		g_object_unref (style);
+	}
+}
+
 char *
 gkbd_indicator_config_get_images_file (GkbdIndicatorConfig *
 				       ind_config,
@@ -248,54 +353,7 @@ gkbd_indicator_config_load_from_gconf (GkbdIndicatorConfig * ind_config)
 		gerror = NULL;
 	}
 
-
-	ind_config->font_family =
-	    gconf_client_get_string (ind_config->conf_client,
-				     GKBD_INDICATOR_CONFIG_KEY_FONT_FAMILY,
-				     &gerror);
-	if (gerror != NULL) {
-		g_warning ("Error reading configuration:%s\n",
-			   gerror->message);
-		ind_config->font_family = g_strdup ("Helvetica");
-		g_error_free (gerror);
-		gerror = NULL;
-	}
-
-	ind_config->font_size =
-	    gconf_client_get_int (ind_config->conf_client,
-				  GKBD_INDICATOR_CONFIG_KEY_FONT_SIZE,
-				  &gerror);
-	if (gerror != NULL) {
-		g_warning ("Error reading configuration:%s\n",
-			   gerror->message);
-		ind_config->font_size = 10;
-		g_error_free (gerror);
-		gerror = NULL;
-	}
-
-	ind_config->foreground_color =
-	    gconf_client_get_string (ind_config->conf_client,
-				     GKBD_INDICATOR_CONFIG_KEY_FOREGROUND_COLOR,
-				     &gerror);
-	if (gerror != NULL) {
-		g_warning ("Error reading configuration:%s\n",
-			   gerror->message);
-		g_error_free (gerror);
-		gerror = NULL;
-	}
-
-	ind_config->background_color =
-	    gconf_client_get_string (ind_config->conf_client,
-				     GKBD_INDICATOR_CONFIG_KEY_BACKGROUND_COLOR,
-				     &gerror);
-	if (gerror != NULL) {
-		g_warning ("Error reading configuration:%s\n",
-			   gerror->message);
-		g_error_free (gerror);
-		gerror = NULL;
-	}
-
-
+	gkbd_indicator_config_load_font_from_gconf (ind_config);
 
 	gkbd_indicator_config_free_enabled_plugins (ind_config);
 	ind_config->enabled_plugins =
@@ -346,7 +404,8 @@ void
 gkbd_indicator_config_activate (GkbdIndicatorConfig * ind_config)
 {
 	xkl_engine_set_secondary_groups_mask (ind_config->engine,
-					      ind_config->secondary_groups_mask);
+					      ind_config->
+					      secondary_groups_mask);
 }
 
 void
@@ -365,5 +424,6 @@ void
 gkbd_indicator_config_stop_listen (GkbdIndicatorConfig * ind_config)
 {
 	gkbd_desktop_config_remove_listener (ind_config->conf_client,
-					     &ind_config->config_listener_id);
+					     &ind_config->
+					     config_listener_id);
 }
